package fr.lri.keller.td8correction;

import android.test.ActivityInstrumentationTestCase2;
import android.test.InstrumentationTestCase;
import android.test.TouchUtils;
import android.test.ViewAsserts;
import android.view.View;
import android.view.ViewGroup;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.RemoteViews;
import android.widget.TextView;

public class TD8ActivityTest extends ActivityInstrumentationTestCase2<TD8Activity> {

    public TD8ActivityTest() {
        super(TD8Activity.class);
    }

    private TD8Activity td8Activity;

    private TextView textViewScore;
    private TextView textViewLevel;
    private Button buttonClick;

    @Override
    protected void setUp() throws Exception {
        super.setUp();

        // Met l'activité en mode tactile afin de pouvoir déclancher dans le code des actions
        // utilisateur
        setActivityInitialTouchMode(true);

        // Récupération de l'activité et de certains des objets de son interface
        td8Activity = getActivity();
        textViewScore = (TextView) td8Activity.findViewById(R.id.textViewScore);
        textViewLevel = (TextView) td8Activity.findViewById(R.id.textViewLevel);
        buttonClick = (Button) td8Activity.findViewById(R.id.buttonClick);
    }

    // Précondition : on a bien accès à l'activité et à tous ses objets dont on a besoin
    public void testPreconditions() {
        assertNotNull("td8Activity == null", td8Activity);
        assertNotNull("textViewScore == null", textViewScore);
        assertNotNull("textViewLevel == null", textViewLevel);
        assertNotNull("buttonClick == null", buttonClick);
    }

    // Vérification : le niveau est le cinquième du score +1
    public void testRelationScoreLevel() {
        int score = Integer.parseInt(textViewScore.getText().toString());
        int level = Integer.parseInt(textViewLevel.getText().toString());
        assertEquals((score / 5) + 1, level);
    }

    // Vérification : le bouton est affiché comme on le souhaite
    public void testButtonClickLayout() {
        final View decorView = td8Activity.getWindow().getDecorView();

        // Vérifie que le bouton est bien dans la vue globale de l'activité
        ViewAsserts.assertOnScreen(decorView, buttonClick);

        // Vérifie certaines propriétés du layout
        final ViewGroup.LayoutParams layoutParams =
                buttonClick.getLayoutParams();
        assertNotNull(layoutParams);
        assertEquals(layoutParams.width, WindowManager.LayoutParams.WRAP_CONTENT);
        assertEquals(layoutParams.height, WindowManager.LayoutParams.WRAP_CONTENT);
    }

    // Vérification : le score augmente de 1 lorsqu'on clique sur le bouton
    public void testButtonClickScoreIncremente() {
        // Récupération du score avant le clic
        int scoreAvant = Integer.parseInt(textViewScore.getText().toString());

        // Simulation d'un clic
        TouchUtils.clickView(this, buttonClick);

        // Récupération du score après le clic
        int scoreApres = Integer.parseInt(textViewScore.getText().toString());

        // Vérification
        assertEquals(scoreAvant + 1, scoreApres);
    }

    // Verification : le score est toujours un entier positif
    public void testScorePositif() {
        int score = Integer.parseInt(textViewScore.getText().toString());
        assertTrue(score >= 0);
    }

    // Verification : le niveau est toujours un entier strictement positif
    public void testLevelStrictementPositif() {
        int level = Integer.parseInt(textViewLevel.getText().toString());
        assertTrue(level > 0);
    }

    // Vérification : le niveau augmente de 1 lorsqu'on clique cinq fois sur le bouton
    public void testButtonClickNiveauIncremente() {
        // Récupération du niveau avant les clics
        int levelAvant = Integer.parseInt(textViewLevel.getText().toString());

        // Simulation de cinq clics
        for (int i = 0; i < 5; i++) {
            TouchUtils.clickView(this, buttonClick);
        }

        // Récupération du niveau après les clics
        int levelApres = Integer.parseInt(textViewLevel.getText().toString());

        // Vérification
        assertEquals(levelAvant + 1, levelApres);
    }

    // Vérification : le niveau augmente de 1 lorsqu'on clique cinq fois sur le bouton
    public void testButtonClickNiveauIncrementeMinimal() {
        // Récupération du score et du niveau avant les clics
        int scoreAvant = Integer.parseInt(textViewScore.getText().toString());
        int levelAvant = Integer.parseInt(textViewLevel.getText().toString());

        // Calcul du nombre minimal de fois à appuyer sur le bouton pour gagner un niveau
        int nbAppui = 5 - (scoreAvant % 5);

        // Simulation de ces clics
        for (int i = 0; i < nbAppui; i++) {
            TouchUtils.clickView(this, buttonClick);
        }

        // Récupération du niveau après les clics
        int levelApres = Integer.parseInt(textViewLevel.getText().toString());

        // Vérification
        assertEquals(levelAvant + 1, levelApres);
    }

}
